#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "QTime"
#include "QTimer"
#include "QSound"
#include "QFile"
#include <stdlib.h>
#include <math.h>
#include <QtCore/qendian.h>
#include <QtMultimedia/QAudioDeviceInfo>
#include <QtMultimedia/QAudioInput>
#include <QtMultimedia/QAudioFormat>
#include <QDebug>
#include <QtGui/QDesktopServices>

#include <QSensor>
#include <QAccelerometer>

#define TIME_BETWEEN_SOUNDS 2

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    m_device(QAudioDeviceInfo::defaultInputDevice())
{


//////////////////////////////////// Accelerometer ///////////////////////////////
    sensorAcc.start();
    connect(&sensorAcc,SIGNAL(readingChanged()),SLOT(readSensorAcc()));

//////////////////////////////////// Orientation /////////////////////////////////
    sensorOri.start();
    connect(&sensorOri,SIGNAL(readingChanged()),SLOT(readSensorOri()));

///////////////////////////// for Sound Reading ///////////////////////////////////////
    prevLevel = 0;
    maxLevel = 2;

    m_pullMode = true;

    m_format.setFrequency(8000);
    m_format.setChannels(1);
    m_format.setSampleSize(16);
    m_format.setSampleType(QAudioFormat::SignedInt);
    m_format.setByteOrder(QAudioFormat::LittleEndian);
    m_format.setCodec("audio/pcm");

    QAudioDeviceInfo info(QAudioDeviceInfo::defaultInputDevice());
    if (!info.isFormatSupported(m_format)) {
        qWarning() << "Default format not supported - trying to use nearest";
        m_format = info.nearestFormat(m_format);
    }

    m_audioInfo  = new AudioInfo(m_format, this);
    connect(m_audioInfo, SIGNAL(update()), this, SLOT(updateLevel()));

    m_audioInput = new QAudioInput(m_device, m_format, this);
    qWarning() << "INFO";
    m_audioInfo->start();
    qWarning() << "input";
    m_audioInput->start(m_audioInfo);

    readLevel = false;
////////////////////////////////////////////////////////////////////////////////
    ui->setupUi(this);
    generalInitialization();

    startClock();
    time_from_start = TIME_BETWEEN_SOUNDS;
    connect(ui->bsetAlarm, SIGNAL(clicked()), this,SLOT(setAlarm()));
    connect(ui->applyButton, SIGNAL(clicked()), this, SLOT(applySettings()));
    connect(ui->resetButton, SIGNAL(clicked()), this, SLOT(resetSettings()));

    //ui->selectedTime->setAttribute();
}


MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::playAlarm()
{
    if(alarm_state == 1)
        alarm_state = 2;
    if(alarm_state == 2) {
        ui->bsetAlarm->setText("Stop Alarm");
        soundObject->setLoops(1);
        soundObject->play();
        alarm_state = 3;
        time_from_start += 0.2;
        QTimer::singleShot(200,this,SLOT(playAlarm()));
    }else if(alarm_state == 3) {

        if(soundObject->isFinished())
        {

            //soundObject->play();
            time_from_start += TIME_BETWEEN_SOUNDS;
            alarm_state = 2;
            if(time_from_start < 60)
                QTimer::singleShot(TIME_BETWEEN_SOUNDS*1000,this,SLOT(playAlarm()));
            else
            {
                time_from_start = 0;
                QTimer::singleShot(60000,this,SLOT(playAlarm())); //trebuie modificat ceva la starea alarmei, pt ca va suna in minutul urmator chiar daca sa activat la alta ora
            }
        }
        else
        {
            time_from_start += 0.1;
            QTimer::singleShot(100,this,SLOT(playAlarm()));
        }
    }
}

void MainWindow::timeControl()
{
    ui->timeLabel->setText(QTime::currentTime().toString());
    QTimer::singleShot(1000,this,SLOT(timeControl()));
}

void MainWindow::startClock()
{
    ui->timeLabel->setText(QTime::currentTime().toString());
    if (alarm_state == 1)
    {
        if(QTime::currentTime().toString() == alarmTime)
        {
            //alarmTime
            initializeAlarm();
            playAlarm();
            ui->bsetAlarm->setText("Stop Alarm");
        }
        if(QTime::currentTime().toString() == recordTime)
        {
            readLevel = true;
            readInitialOri();
            QTimer::singleShot(3000,this,SLOT(setLevelAverage()));
        }
    }

    QTimer::singleShot(1000,this,SLOT(startClock()));
}

void MainWindow::setAlarm()
{
  if (ui->bsetAlarm->text() == "Set Alarm")
  {
      ui->selectedTime->setReadOnly(true);
      ui->bsetAlarm->setText("Unset Alarm");
      setAlarmTime();

  }else  if (ui->bsetAlarm->text() == "Unset Alarm")
  {
      ui->selectedTime->setReadOnly(false);
      ui->bsetAlarm->setText("Set Alarm");
      unsetAlarmTime();
  }else  if (ui->bsetAlarm->text() == "Stop Alarm")
  {
      ui->selectedTime->setReadOnly(false);
      soundObject->stop();
      ui->bsetAlarm->setText("Set Alarm");
      unsetAlarmTime();
  }

}

void MainWindow::updateLevel(){
    qreal currentLevel = m_audioInfo->level();

    if(readLevel)
        if(prevLevel < currentLevel)
            prevLevel = currentLevel;

    if(currentLevel < maxLevel){
        ui->label_trigger->setText("max:"+QString::number(float(maxLevel)));
        ui->label_current->setText("Curent: "+QString::number(float(currentLevel)));
    }else
    {
        ui->label_current->setText("Reached "+QString::number(float(currentLevel)));
        if(conf.getStoppBySound())
        if(alarm_state == 2 || alarm_state == 3)
        {
        if(soundObject->isFinished())
        {
            unsetAlarmTime();
            soundObject->stop();
            ui->bsetAlarm->setText("Set Alarm");
            ui->selectedTime->setReadOnly(false);
            qWarning("alarm stopped by sound");
        }
        }
    }

    ui->soundBar->setValue(int(float(currentLevel)*100));


}

void MainWindow::setLevelAverage(){
    if(prevLevel )
    maxLevel = prevLevel*1.2;
    if (maxLevel > 1)
        maxLevel = 1;
    if (maxLevel < 0.5)
        maxLevel = maxLevel*1.5;
    if (maxLevel < 0.15)
        maxLevel = 0.15;
    readLevel = false;
}

void MainWindow::applySettings()
{
    conf.setSoundFileName(ui->soundComboBox->itemText(ui->soundComboBox->currentIndex()));
    conf.setStoppBySound(ui->stopBySound->isChecked());
    conf.setStoppByAccelerometer(ui->stopByShake->isChecked());
    conf.setStoppByOrientation(ui->stopByFlip->isChecked());
    conf.setSensitivityAccelerometer(ui->sensitivitySlider->value());
    initializeAlarm();
}

void MainWindow::resetSettings()
{
    ///////////////// set check boxes //////////////
    ui->stopBySound->setChecked(conf.getStoppBySound());
    ui->stopByShake->setChecked(conf.getStoppByAccelerometer());
    ui->stopByFlip->setChecked(conf.getStoppByOrientation());
    ui->sensitivitySlider->setValue(conf.getSensitivityAccelerometer());

    ///////////////// set combo box ////////////////
    for(int i=0;i<=ui->soundComboBox->count();i++)
    {
        if(conf.getSoundFileName() == ui->soundComboBox->itemText(i))
            ui->soundComboBox->setCurrentIndex(i);
    }
}

void MainWindow::initializeAlarm()
{
    soundObject = new QSound(conf.getSoundFileName());

    if(!QFile::exists(conf.getSoundFileName())){
        QString musicDirectory = QDesktopServices::storageLocation(QDesktopServices::MusicLocation);
        qWarning() << "Music directory: "+musicDirectory;
        conf.setSoundFileName(musicDirectory+"/"+conf.getSoundFileName());
        soundObject = new QSound(conf.getSoundFileName());
        if(!QFile::exists(conf.getSoundFileName())){
            soundPath = QFileDialog::getOpenFileName(this, tr("Open File"), QDir::currentPath());
            conf.setSoundFileName(soundPath);
            soundObject = new QSound(conf.getSoundFileName());
            ///////////////// set combo box ////////////////
            ui->soundComboBox->addItem(soundPath, QVariant::Color);
            for(int i=0;i<=ui->soundComboBox->count();i++)
            {
                if(conf.getSoundFileName() == ui->soundComboBox->itemText(i))
                    ui->soundComboBox->setCurrentIndex(i);
            }
            qWarning() << conf.getSoundFileName();

        }
    }

}

void MainWindow::generalInitialization()
{
    ui->testLabel->hide();
    ui->label_trigger->hide();
    ui->label_current->hide();
    /////// Initialize the ComboBox //////////
    ui->soundComboBox->addItem("sounds/ringin.wav", QVariant::Color);
    ui->soundComboBox->addItem("sounds/warning.wav", QVariant::Color);
    int i;
    for(i=0;i<=ui->soundComboBox->count();i++)
    {
        if(conf.getSoundFileName() == ui->soundComboBox->itemText(i))
        {
            ui->soundComboBox->setCurrentIndex(i);
            qWarning() << "current combobox item: "+ui->soundComboBox->itemText(i);
            break;
        }
    }
    if(conf.getSoundFileName() != (ui->soundComboBox->itemText(ui->soundComboBox->currentIndex())))
    {
        ui->soundComboBox->addItem(conf.getSoundFileName(), QVariant::Color);
        for(i=0;i<=ui->soundComboBox->count();i++)
        {
            if(conf.getSoundFileName() == ui->soundComboBox->itemText(i))
            {
                //qWarning() << "current combobox item: "+ui->soundComboBox->itemText(i);
                ui->soundComboBox->setCurrentIndex(i);
            }
        }
        //qWarning() << "new combobox item: "+ui->soundComboBox->itemText(i);
    }

    ////////// Initialize Alarm Time ////////
    ui->selectedTime->setTime(QTime::fromString(conf.getTime()));
    /////////////////////////////////////////
    resetSettings();
}

void MainWindow::setAlarmTime()
{
      alarm_state = 1;
      conf.setTime(ui->selectedTime->time().toString());
      alarmTime = ui->selectedTime->time().toString();
      ui->testLabel->setText(alarmTime);

      ////////////// setting record time ////////////////
      QString hour1 = (alarmTime.at(0));
      QString hour2 = (alarmTime.at(1));
      QVariant hour(hour1+hour2);
      int ihour = hour.toInt();
      QString sminutes1 = (alarmTime.at(3));
      QString sminutes2 = (alarmTime.at(4));
      QVariant minutesvariant(sminutes1+sminutes2);
      int minutes = minutesvariant.toInt();
      if(minutes != 0)
          minutes--;
      else
      {
          minutes=59;
          if(ihour!=0)
              ihour--;
          else
              ihour=23;
      }

      QVariant finalMinutes(minutes);
      QVariant fhour(ihour);

      if(minutes < 10)
          finalMinutes.setValue("0"+finalMinutes.toString());

      if(ihour<10)
          fhour.setValue("0"+fhour.toString());

      recordTime = fhour.toString()+":"+finalMinutes.toString()+":"+"56";
      ///////////////////////////////////////////////////////

      ui->testLabel->setText(recordTime);
}

void MainWindow::unsetAlarmTime()
{
    alarm_state = 0;
    ui->testLabel->setText("");
    maxLevel = 1;
    ui->label_trigger->setText("Max: "+QString::number(float(maxLevel)));
}

void MainWindow::readSensorAcc()
{
    accX = sensorAcc.reading()->x();
    accY = sensorAcc.reading()->y();
    accZ = sensorAcc.reading()->z();

    QVariant accXv(accX);
    QVariant accYv(accY);
    QVariant accZv(accZ);

    if(conf.getStoppByAccelerometer())
    if(alarm_state == 2 || alarm_state == 3)
    {
        QVariant sensval(conf.getSensitivityAccelerometer());
        ui->testLabel->setText(sensval.toString());
        QString das = sensval.toString();
        if(qAbs(accXv.toInt()) > (5*conf.getSensitivityAccelerometer()) || qAbs(accYv.toInt())> (5*conf.getSensitivityAccelerometer())
            || qAbs(accZv.toInt())> (5*conf.getSensitivityAccelerometer()))
        {
            unsetAlarmTime();
            soundObject->stop();
            ui->bsetAlarm->setText("Set Alarm");
            ui->selectedTime->setReadOnly(false);
            qWarning("alarm stopped by shake");
        }
    }
}

void MainWindow::readSensorOri()
{
    qreal oriCurrent = sensorOri.reading()->orientation();

    QVariant oriV(oriCurrent);

    ui->testLabel->setText(oriV.toString());

    if(conf.getStoppByOrientation())
    if(alarm_state == 2 || alarm_state == 3)
    {
        if( (ori%2 == 0 && oriV.toInt()+1 == ori)
            ||(ori%2 != 0 && oriV.toInt()-1 == ori ))
        {
            unsetAlarmTime();
            soundObject->stop();
            ui->bsetAlarm->setText("Set Alarm");
            ui->selectedTime->setReadOnly(false);
            qWarning("alarm stopped by flip");
        }

    }
}
void MainWindow::readInitialOri()
{
    qreal oriInit = sensorOri.reading()->orientation();

    QVariant oriInitv(oriInit);
    ori = oriInitv.toInt();
    qWarning() << "Initial Orientation: "+oriInitv.toString();
}

